import java.awt.Shape;
import java.awt.geom.AffineTransform;


/**
 * Abstrakte Klasse Moebel
 * 
 * Die Modellierung ist bei "loesche" anders als im ersten Entwurf,
 * da hier in jedem Fall einfach neu gezeichnet wird.
 * @author  claus albowski
 */

public abstract class Moebel
{
  protected int xPosition;
  protected int yPosition;
  protected int orientierung;
  protected String farbe;
  protected boolean istSichtbar;
  protected int breite;
  protected int tiefe;

  /**
   * Berechnet das zu zeichnende Shape anhand der gegebenen Daten
   */
  protected abstract Shape gibAktuelleFigur();
  
  /**
   * Transformiert das Shape, so dass es die richtige Position
   * und Orientierung einnimmt.
   */
  protected Shape transformiere(Shape shape)
  {
      return  gibTransform().createTransformedShape(shape);
  }
    
  /**
   * Stellt die Transformation zur Verfgung.
   */
  protected AffineTransform gibTransform()
  {
      AffineTransform t = new AffineTransform();
      t.rotate(Math.toRadians(orientierung),gibMitteX(),gibMitteY());
      t.translate(xPosition, yPosition);
      return  t;
  }
  
  /**
   * zeigt die Sichtbarkeit an.
   */
  public boolean gibSichtbarkeit(){
    return istSichtbar;
  }

  /**
   * zeigt die Farbe an.
   */
  public String gibFarbe(){
    return farbe;
  }

  /**
   * Hole die X-Koordinate des Mittelpunktes
   * [ Hilfsfunktion fr das Drehen. Kann bei einfachen Objekten private sein.]
   */
  private int gibMitteX(){
    return xPosition+breite/2;
  }

  /**
   * Hole die Y-Koordinate des Mittelpunktes
   * [ Hilfsfunktion fr das Drehen. Kann bei einfachen Objekten private sein.]
   */
  private int gibMitteY()
  {
    return yPosition+tiefe/2;
  }

  /**
   * Mache dieses Objekt sichtbar. Wenn es bereits sichtbar ist, tue
   * nichts.
   */
  public void zeige()
  {
    istSichtbar = true;
  }

  /**
   * Mache dieses Objekt unsichtbar. Wenn es bereits unsichtbar ist, tue
   * nichts.
   */
  public void verberge()
  {
    istSichtbar = false;
  }

  /**
   * Drehe auf den angegebenen Winkel
   */
  public void dreheAuf(int neuerWinkel)
  {
     orientierung = neuerWinkel;
  }

  /**
   * Bewege dieses Objekt horizontal um 'entfernung' Bildschirmpunkte.
   */
  public void bewegeHorizontal(int entfernung)
  {
    xPosition += entfernung;
  }

  /**
   * Bewege dieses objekt vertikal um 'entfernung' Bildschirmpunkte.
   */
  public void bewegeVertikal(int entfernung)
  {
    yPosition += entfernung;
  }

 
  /**
   * ndere die Farbe dieses Objektes in 'neueFarbe'.
   * Gltige Angaben sind "rot", "gelb", "blau", "gruen",
   * "lila" und "schwarz".
   */
  public void aendereFarbe(String neueFarbe)
  {
    farbe = neueFarbe;
  }

}
