import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;
import java.awt.Shape;
import java.awt.geom.Point2D;

/**
 * Die Klasse Moeblierung
 * bernimmt die Daten der Mbel, soweit sie fr die Leinwand notwendig sind.
 * Aus Grnden der Vereinfachung sind keine Mbel - Gruppen eingebaut!
 * @author (claus albowski) 
 * @version (Feb 06)
 */
public class Moeblierung
{
    private List moebelListe;
    private Iterator it;
    private Object aktuell, ausgewaehlt;
    
    /**
     * Konstruktor
     */
    public Moeblierung(){
        moebelListe=new ArrayList();
        aktuell=ausgewaehlt=null;
    }
    
    /**
     * prft, ob ein Mbel schon enthalten ist.
     */
    private boolean enthalten(Moebel moebel){
        for (Iterator it=moebelListe.iterator();it.hasNext();){
            if (it.next().equals(moebel))  return true;
        }
        return false;
    }

    /**
     * nimmt das entsprechende Moebel in die Liste auf und zeichnet neu.
     * Es wird verhindert, dass ein schon eingebautes Mbel erneut eingebaut wird.
     * Dabei wird aber nur Objektgleichheit berprft.
     */
    public void nimmAuf(Moebel moebel)
    {
        if (!enthalten(moebel)){
            moebelListe.add(moebel); // am Ende hinzufgen
            start();
        }
    }

    /**
     * Entferne das gegebene Moebel aus der Mblierung.
     * @param  <figur>  die Figur, die entfernt werden soll
     */
    public void entferne(Moebel moebel)
    {
        moebelListe.remove(moebel); // entfernen,falls schon eingetragen
        start();
    }
    
    /**
     * setzt den Iterator auf den Anfang der Moebelliste zurck.
     */
    public void start(){
        it=moebelListe.iterator();
    }
    
    /**
     * setzt den Iterator auf den Anfang der Moebelliste zurck.
     */
    public boolean amEnde(){
        return !(it.hasNext());
    }
    
    /**
     * setzt auf das nchste Element weiter.
     */
    public void weiter(){
        if (it.hasNext()) aktuell=it.next(); else aktuell=null;
    }
    
    /**
     * gibt das Shape des aktuellen Mbels zurck.
     */
    public Shape aktuellesShape(){
        if (aktuell!=null) return ((Moebel)aktuell).gibAktuelleFigur(); else return null;
    }
    
    /**
     * gibt die Farbe des aktuellen Mbels zurck.
     */
    public String aktuelleFarbe(){
        if (aktuell!=null) return ((Moebel)aktuell).gibFarbe(); else return null;
    }
    
    /**
     * setzt die Eigenschaft ausgewhlt zu sein fr das aktuelle Moebel.
     * (Fr Bearbeitung)
     */
    public void waehleAus(){
        ausgewaehlt=aktuell;
    }
    
    /**
     * gibt zurck, ob das aktuelle Mbel ausgewhlt ist.
     * (Fr Bearbeitung)
     */
    public boolean istAusgewaehlt(){
        return (aktuell==ausgewaehlt);
    }
    
    /**
     * gibt zu einem mit den Koordinaten bergebenen Punkt das erste Mbel,
     * das ihn enthlt.
     */
    public Moebel gibMoebelZuPunkt(int x, int y){
        Point2D.Double p = new Point2D.Double(x,y);
        for (Iterator it=moebelListe.iterator();it.hasNext();){
            Moebel temp = (Moebel)it.next();
            if (temp.gibAktuelleFigur().contains(p)) return temp;
        }
        return null;
    }
    
    /**
     * verwandleInInt
     * wandelt den String -wenn mglich- in eine positive int - Zahl um.
     */
    private int verwandleInInt(String s)
      throws NumberFormatException {
        return Integer.parseInt(s, 10);
    }
    
    /**
     * erzeugt ein neues Mbel vom bergebenen Typ.
     */
    public boolean neuesMoebel(String typ,
                               String xS,
                               String yS,
                               String farbe,
                               String oS,
                               String bS,
                               String tS){
        int x,y,orientierung,breite,tiefe;
        try {
            x=verwandleInInt(xS);
            y=verwandleInInt(yS);
            orientierung=verwandleInInt(oS);
            breite=verwandleInInt(bS);
            tiefe=verwandleInInt(tS);
        } catch (NumberFormatException exc) {return false;}
        if (x<0) return false;
        if (y<0) return false;
        if (breite<=0) return false;
        if (tiefe<=0) return false;
        if (typ.equals("Stuhl")){
            nimmAuf(new Stuhl(x,y,farbe,orientierung,breite,tiefe));
            return true;
        }
        else if (typ.equals("Tisch")){
            nimmAuf(new Tisch(x,y,farbe,orientierung,breite,tiefe));
            return true;
        }
        else if (typ.equals("Schrank")){
            nimmAuf(new Schrank(x,y,farbe,orientierung,breite,tiefe));
            return true;
        }
        else if (typ.equals("Bett")){
            nimmAuf(new Bett(x,y,farbe,orientierung,breite,tiefe));
            return true;
        }
        return false;
    }
}
