import javax.swing.JPanel;

import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.Color;
import java.awt.Dimension;

/************************************************************************
* Die Klasse Zeichenflaeche - die Klasse fr die GUI-Komponente,
* die tatschlich im Fenster angezeigt wird. Diese Klasse
* definiert ein JPanel mit der zustzlichen Mglichkeit, das auf ihm
* gezeichnet Image aufzufrischen (erneut zu zeichnen).
*/
public class Zeichenflaeche extends JPanel{
    private Graphics2D graphic;
    private Image offScreenImage;
    private Color hintergrundfarbe=Color.white;
    private int width,height;

    public Zeichenflaeche(int width, int height){
        this.width=width;
        this.height=height;
    }
    

    public void changed (Graphics g) 
    {
        paint(g);
    } 
 
    public void paint(Graphics g)
    {
      g.drawImage(offScreenImage, 0, 0, null);
    }

      
    /**
     * Zeichne die Shapes auf die Zeichenflche.
     */    
    public void zeichne(Moeblierung moeblierung){
        moeblierung.start();
        loeschen();
        while (!moeblierung.amEnde()){
            moeblierung.weiter();
            (new ShapeMitFarbe
            (moeblierung.aktuellesShape(),
             moeblierung.aktuelleFarbe(),
             moeblierung.istAusgewaehlt())).draw(graphic);
        }
    }
    
    /**
     * Lsche die gesamte Zeichenflaeche.
     */
    public void loeschen()                    // Sichtbarkeit gendert !
    {
        warte(100);
        if (graphic == null)
        {
            // Erstmaliger Aufruf: erzeuge das Bildschirm-Image fr die
            // Doppelpufferung und flle es mit der Hintergrundfarbe.
            offScreenImage = createImage(width, height);
            graphic = (Graphics2D) getParent().getGraphics();
            graphic.setColor(Color.black);
        }
        Color original = graphic.getColor();
        graphic.setColor(hintergrundfarbe);
        graphic.fill(new Rectangle(0, 0, width, height));
        graphic.setColor(original);
    }

  /**
   * Warte fr die angegebenen Millisekunden.
   * Mit dieser Operation wird eine Verzgerung definiert, die
   * fr animierte Zeichnungen benutzt werden kann.
   * @param  millisekunden die zu wartenden Millisekunden
   */
  public void warte(int millisekunden)
  {
    try { Thread.sleep(millisekunden); }
    catch (Exception e) { }
  }


/************************************************************************
* Interne Klasse ShapeMitFarbe - Da die Klasse Shape des JDK nicht auch
* eine Farbe mitverwalten kann, muss mit dieser Klasse die Verknpfung
* modelliert werden. Zustzlich wird verwaltet, ob das Objekt ausgewhlt
* ist, was hier durch geflltes Zeichnen dargestellt wird.
*/
    private class ShapeMitFarbe
    {
        private Shape shape;
        private String farbe;
        private boolean ausgewaehlt;

        public ShapeMitFarbe(Shape shape, String farbe, boolean ausgewaehlt)
        {
            this.shape = shape;
            this.farbe = farbe;
            this.ausgewaehlt = ausgewaehlt;
        }

        /**
         * Setze die Zeichenfarbe der Zeichenflaeche.
         * @param  farbname der Name der neuen Zeichenfarbe.
         */
        public void setzeZeichenfarbe(String farbname)
        {
            if (farbname.equals("rot")) graphic.setColor(Color.red);
            else if (farbname.equals("schwarz")) graphic.setColor(Color.black);
            else if (farbname.equals("blau")) graphic.setColor(Color.blue);
            else if (farbname.equals("gelb")) graphic.setColor(Color.yellow);
            else if (farbname.equals("gruen")) graphic.setColor(Color.green);
            else if (farbname.equals("lila")) graphic.setColor(Color.magenta);
            else if (farbname.equals("weiss")) graphic.setColor(Color.white);
            else graphic.setColor(Color.black);
        }
            
        /**
         * zeichnet mit Farbe und dabei ausgefllt oder nicht
         * abhngig davon, ob das Shape ausgewhlt ist oder nicht.
         */
        public void draw(Graphics2D graphic)
        {
            setzeZeichenfarbe(farbe);
            if (ausgewaehlt) graphic.fill(shape);
            else graphic.draw(shape);
        }
    }
}